--[[
config = make_config{
	name = 'killshot',
	{var_name = 'ikill', type = 'boolean', desc = 'You killed someone.', default = true},
	
}
]]
local function read_value(section, name, default)
	local val = gkini.ReadString(section, name, '__nil__')
	if val == '__nil__' then 
		return default
	else
		local type, val = val:match('(.-):(.*)')
		if type == 'number' then 
			return tonumber(val)
		elseif type == 'string' then
			return val
		elseif type == 'boolean' then 
			return val=='true'
		end
	end
end

local function write_value(section, name, value)
	local val = ('%s:%s'):format(type(value), tostring(value))
	gkini.WriteString(section, name, val)
end

local cli_handlers = {
	boolean = function(option, state, args)
		if not option.is_main and not args[1] then
			local current = state and 'on' or 'off'
			print(('%s is currently set to %s.'):format(option.var_name, current))
		elseif args[1]=='on' or args[1]=='off' then
			local name = option.is_main and option.plugin_name or option.var_name
			print(('%s is %s.'):format(name, args[1]))
			return args[1]=='on'
		else
			print(('Wrong argument to %s, expected "on" or "off".'):format(option.var_name))
		end
	end,
	number = function(option, state, args)
		if not args[1] then
			print(('%s is currently set to %d'):format(option.var_name, state))
		elseif tonumber(args[1]) then
			local new_state = tonumber(args[1])
			print(('%s is now set to %s'):format(option.var_name, new_state))
			return new_state
		else
			print(('Wrong argument to %s, expected a number.'):format(option.var_name))
		end
	end,
	string = function(option, state, args)
		if not args[1] then
			print(('%s is currently set to %s'):format(option.var_name, state))
		else
			local new_state = table.concat(args, ' ')
			print(('%s is now set to "%s"'):format(option.var_name, new_state))
			return new_state
		end
	end,
}

local function print_command_list(options)
	if options.main then
		print(('%s %s is %s'):format(options.name, options.version or '', options.config['on'] and 'enabled' or 'disabled'))
		print('Available commands:')
		print('on/off - to enable or disable the plugin')
	else
		print(('%s %s - Available commands:'):format(options.name, options.version or ''))
	end
	for i,option in ipairs(options) do
		if not option.hidden and not option.is_main then
			local current = options.config[option.var_name]
			if type(current) == 'boolean' and not option.is_main then 
				current = current and 'on' or 'off'
			elseif type(current) == 'string' then 
				current = ('"%s"'):format(current)
			end
			local option_help = ('%s - %s. %s'):format(option.var_name, current, option.desc or '')
			-- local option_help = option.var_name .. ' - ' .. (option.desc or '')
			-- option_help = option_help .. ' Currently set to: '.. tostring(current)
			-- option_help = option_help .. ' Default: '.. tostring(option.default) -- prints true instead of on/off for bools,
			print(option_help)
		end
	end
	print('resets [option] - reverts option to it\'s default value') -- ya i'm being really lazy here...
end

local global_commands = { -- this could work like the config description table does... since it kinda needs to be printed in list of options anyway.... actually main list needs to become generic so that there can be sub-menus or maybe the whole thing is just no good...
	reset = function(options, commands, data) --yaaaah this sucks, would be nice if each 'option' was object-like with it's own Set function or something, not to mention easy access to it's current value, so only that 'object' needs to be passed
		if commands[data[1]] then
			local default = commands[data[1]].default
			options.config.Set(data[1], default)
			print(('%s has be reset to it\'s default value: %s'):format(data[1], default))
		else
			print('No such option')
		end
	end
}
local function make_config(options)
	assert(tostring(options.name), 'config name not given or not a string')
	options.config = {}
	local commands = {}
	-- local main
	for i,option in ipairs(options) do
		assert(option.var_name, ('var_name missing for option %d'):format(i))
		if option.default==nil then print(('default missing for %s option %s'):format(options.name, option.var_name)) end
		if option.is_main then 
			options.main = option
			options.main.plugin_name = options.name 
		end
		-- option.config = config
		option.cli_handler = option.handler or cli_handlers[option.type]
		commands[option.var_name] = option
		options.config[option.var_name] = read_value(options.name, option.var_name, option.default)
	end
	
	
	RegisterUserCommand(options.name, function(_,data)
		if not data or not data[1] then
			print_command_list(options)
		else
			local option
			if data[1] == 'on' or data[1] == 'off' then
				option = options.main
			else
				local command = table.remove(data, 1):lower()
				if global_commands[command] then
					global_commands[command](options, commands, data)
					return
				end
				option = commands[command]
			end
			if option then
				local state = options.config[option.var_name]
				local new_state = option.cli_handler(option, state, data)
				if new_state ~= nil then
					options.config[option.var_name] = new_state
					write_value(options.name, option.var_name, new_state)
				end
			else
				print'Wrong command.'
				print_command_list(options)
			end
		end
	end)
	options.config.Set = function(var_name, value)
		write_value(options.name, var_name, value)
	end
	options.config.options = options --so that the original description table is visible by the plugin (not really sure what for yet), btw - yay circular reference
	return options.config
end

return make_config